﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Web;
using System.Net.Mime;
using System.Reflection;

namespace Umc.Core.Web.HttpCompression
{
	/// <summary>
	/// Umc.Core<br/>
	/// HTTP 압축 스트림을 지원합니다.
	/// </summary>
	public class HttpCompression : IHttpCompression
	{
		#region Members
		private static readonly string FILENAME_SCRIPTRESOURCE	= "scriptresource.axd";			// ScriptResource.axd 파일명
		private static readonly string KEY_UMC_CORE				= "Powered By Umc.Core";
		private static readonly string VALUE_UMC_CORE			= "http://powerumc.kr";
		private IHttpCompressionAlgorism algorism;
		private static FieldInfo _HttpWriterFieldInfo = typeof(HttpResponse).GetField("_httpWriter", BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance);
        private static FieldInfo _IgnoreFurtherWritersFieldInfo = typeof(HttpWriter).GetField("_ignoringFurtherWrites", BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance);
		#endregion

		#region Constructor
		/// <summary>
		/// Umc.Core<br/>
		/// HTTP 압축 스트림을 지원하는 클래스를 생성합니다.
		/// </summary>
		/// <param name="algorism">
		/// <see cref="Umc.Core.Web.HttpCompression.IHttpCompressionAlgorism"/> 인터페이스를 구현한 알고리즘입니다.
		/// </param>
		public HttpCompression(IHttpCompressionAlgorism algorism)
		{
			this.algorism	= algorism;
		} 
		#endregion

		#region IHttpCompression 멤버
		/// <summary>
		/// Umc.Core<br/>
		/// HTTP 압축을 수행합니다.
		/// </summary>
		/// <param name="context">HTTP 파이프라인의 HttpContext 입니다.</param>
		/// <param name="level">압축율을 지정합니다.</param>
		public virtual void Compression(HttpContext context, HttpCompressionLevel level)
		{
			// 브라우져가 알고리즘에 대해 지원하지 않으면 압축하지 않는다
			if( !SupportedBrowser(context, algorism) ) return;
			if (context.Request.FilePath.ToLower().EndsWith(FILENAME_SCRIPTRESOURCE))
			{
				// 웹리소스 요청의 d 파라메터의 복호화 결과가 Z 로 시작하면 gzip 압축이므로 압축을 수행하지 않는다.
				string param	= context.Request.QueryString["d"];
				string decrypt	= HttpUtil.DecryptString(param);
				if( decrypt.ToUpper().StartsWith("Z") )
					return;
			}

			if (context.CurrentHandler is System.Web.Handlers.AssemblyResourceLoader)
			{
				// Send Content 가 리소스일때 Content-Type 이 무조건 text/html 로 전송되는 문제가 있다.
				// 때문에 리소스의 Response 를 현재의 파이프라인의 Context Response 로 교체한다.
			    HttpWriter httpWriter = (HttpWriter)_HttpWriterFieldInfo.GetValue(context.Response);
			    _IgnoreFurtherWritersFieldInfo.SetValue(httpWriter, false);
			}

			context.Response.Filter = algorism.CompressionAlgorism(context.Response.Filter, level);
			
			SetHeader(context);

			
		}

		#endregion

		/// <summary>
		/// Umc.Core<br/>
		/// 사용자의 브라우져가 Algorism Provider 가 지원하는 압축 알고리즘인지의 여부를 판단합니다.
		/// </summary>
		/// <param name="context">현재의 HttpContext 개체입니다.</param>
		/// <param name="algorism">압축 알고리즘 입니다.</param>
		/// <returns>사용자의 브라우져가 압축 Algorism 지원하면 True 이고 그렇지 않으면 False 입니다.</returns>
		public static bool SupportedBrowser(HttpContext context, IHttpCompressionAlgorism algorism)
		{
			return context.Request.Headers["Accept-Encoding"].Contains(algorism.ContentEncoding);
		}

		/// <summary>
		/// Umc.Core<br/>
		/// 압축을 수행한 후 마지막으로 호출됩니다. 사용자에게 Content-Encoding 을 알리도록 합니다.
		/// </summary>
		/// <param name="context">현재의 HttpContext 개체입니다.</param>
		private void SetHeader(HttpContext context)
		{
			context.Response.AppendHeader("Content-Encoding", algorism.ContentEncoding);
			context.Response.AppendHeader(KEY_UMC_CORE, VALUE_UMC_CORE);
		}
	}
}
