﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;

namespace Umc.Core.Web.HttpCompression
{
	/// <summary>
	/// Umc.Core<br/>
	/// HTTP 압축을 지원하는 팩토리입니다.
	/// </summary>
	public sealed class HttpCompressionFactory
	{
		#region Private constructor
		private HttpCompressionFactory()
		{
		} 
		#endregion

		#region CreateFactory
		/// <summary>
		/// Umc.Core<br/>
		/// HTTP 압축을 지원하는 <see cref="Umc.Core.Web.HttpCompression.IHttpCompression"/> 인터페이스를 가져옵니다.
		/// </summary>
		/// <param name="type">압축 알고리즘 입니다.</param>
		/// <returns>
		/// HTTP 압축을 지원하는 <see cref="Umc.Core.Web.HttpCompression.IHttpCompression"/> 인터페이스를 가져옵니다.
		/// </returns>
		/// <example>
		/// Factory 메서드의 사용방법입니다. 일반적으로 HttpModule 에 등록하여 사용하도록 합니다.<br/>
		/// 만약, 특정 페이지에서만 적용되는 압축이나 암호화 보안이 필요하면(Custom Provider) 다음과 같이 사용할 수 있습니다.
		/// <code>
		/// HttpCompressionFactory.CreateFactory(HttpCompressionType.GZip).Compression(context,HttpCompressionLevel.High);
		/// </code>
		/// </example>
		public static IHttpCompression CreateFactory(HttpCompressionType type)
		{
			IHttpCompression compression = null;

			switch (type)
			{
				case HttpCompressionType.GZip:
					compression = new HttpCompression(new GZipCompression());
					break;

				case HttpCompressionType.Deflate:
					compression = new HttpCompression(new DeflateCompression());
					break;

				default:
					throw new NotSupportedException("type");
			}

			return compression;
		}

		/// <summary>
		/// Umc.Core<br/>
		/// Comtom Provider 를 통해 HTTP 압축을 지원하는 <see cref="Umc.Core.Web.HttpCompression.IHttpCompression"/> 인터페이스를 가져옵니다.
		/// </summary>
		/// <param name="provider">
		/// 압축 알고리즘을 구현한 <see cref="Umc.Core.Web.HttpCompression.IHttpCompressionAlgorism"/> 입니다.
		/// </param>
		/// <returns>HTTP 압축을 지원하는 <see cref="Umc.Core.Web.HttpCompression.IHttpCompression"/> 인터페이스를 가져옵니다.</returns>
		internal static IHttpCompression CreateFactory(string provider)
		{
			return (IHttpCompression)Activator.CreateInstance(typeof(HttpCompression),
					Activator.CreateInstance(Type.GetType(provider), null) as IHttpCompressionAlgorism);
		} 
		#endregion
	}
}
